#include "BeschreibungEditor.h"
#include <Datei.h>
#include <DateiSystem.h>
#include <Bild.h>

// Inhalt der BeschreibungEditor Klasse aus BeschreibungEditor.h
// Konstruktor
BeschreibungEditor::BeschreibungEditor(const char* pf)
	: ReferenceCounter()
{
	pfad = pf;
	pfad += "/live/map/client";
	fehler = "";
}

// Destruktor
BeschreibungEditor::~BeschreibungEditor()
{}

// nicht constant
bool BeschreibungEditor::prozessMessage(SKlient* zKlient)
{
	char message = 0;
	zKlient->getNachrichtEncrypted(&message, 1);
	switch (message)
	{
	case 0x1: // Beschreibung laden
	{
		Text p = pfad.getText();
		p += "/beschreibung.ksgs";
		if (!DateiExistiert(p))
		{
			fehler = "Die Beschreibung wurde nicht gefunden.";
			return 0;
		}
		zKlient->sendeEncrypted("\1", 1);
		Datei d;
		d.setDatei(p);
		bool ok = d.open(Datei::Style::lesen);
		char* buffer = new char[2048];
		int gr = (int)d.getSize();
		zKlient->sendeEncrypted((char*)&gr, 4);
		while (gr > 0)
		{
			int l = gr > 2048 ? 2048 : gr;
			d.lese(buffer, l);
			zKlient->sende(buffer, l);
			gr -= l;
		}
		delete[] buffer;
		d.close();
		if (!ok)
		{
			fehler = "Es ist ein Fehler beim laden der Beschreibung aufgetreten.";
			return 0;
		}
		zKlient->sendeEncrypted("\1", 1);
		return 1;
	}
	case 0x2: // Titelbild laden
	{
		Text p = pfad.getText();
		p += "/titel.ltdb";
		if (!DateiExistiert(p))
		{
			fehler = "Das Titelbild wurde nicht gefunden.";
			return 0;
		}
		LTDBDatei dat;
		dat.setDatei(dynamic_cast<Text*>(p.getThis()));
		dat.leseDaten(0);
		Bild* b = dat.laden(0, dat.zBildListe()->get(0));
		if (!b)
		{
			fehler = "Das Titelbild wurde nicht gefunden.";
			return 0;
		}
		zKlient->sendeEncrypted("\1", 1);
		int br = b->getBreite();
		int hi = b->getHeight();
		zKlient->sendeEncrypted((char*)&br, 4);
		zKlient->sendeEncrypted((char*)&hi, 4);
		char* buffer = (char*)b->getBuffer();
		__int64 gr = br * hi * 4;
		for (int i = 0, l = gr > 2048 ? 2048 : (int)gr; gr > 0; i += l, gr -= l, l = gr > 2048 ? 2048 : (int)gr)
			zKlient->sende(&(buffer[i]), l);
		b->release();
		return 1;
	}
	case 0x3: // Minimap laden
	{
		Text p = pfad.getText();
		p += "/minimap.ltdb";
		if (!DateiExistiert(p))
		{
			fehler = "Das Minimap Bild wurde nicht gefunden.";
			return 0;
		}
		LTDBDatei dat;
		dat.setDatei(dynamic_cast<Text*>(p.getThis()));
		dat.leseDaten(0);
		Bild* b = dat.laden(0, dat.zBildListe()->get(0));
		if (!b)
		{
			fehler = "Das Minimap Bild wurde nicht gefunden.";
			return 0;
		}
		zKlient->sendeEncrypted("\1", 1);
		int br = b->getBreite();
		int hi = b->getHeight();
		zKlient->sendeEncrypted((char*)&br, 4);
		zKlient->sendeEncrypted((char*)&hi, 4);
		char* buffer = (char*)b->getBuffer();
		__int64 gr = br * hi * 4;
		for (int i = 0, l = gr > 2048 ? 2048 : (int)gr; gr > 0; i += l, gr -= l, l = gr > 2048 ? 2048 : (int)gr)
			zKlient->sende(&(buffer[i]), l);
		b->release();
		return 1;
	}
	case 0x4: // Ladebild laden
	{
		Text p = pfad.getText();
		p += "/ladebild.ltdb";
		if (!DateiExistiert(p))
		{
			fehler = "Das Ladebild wurde nicht gefunden.";
			return 0;
		}
		LTDBDatei dat;
		dat.setDatei(dynamic_cast<Text*>(p.getThis()));
		dat.leseDaten(0);
		Bild* b = dat.laden(0, dat.zBildListe()->get(0));
		if (!b)
		{
			fehler = "Das Ladebild wurde nicht gefunden.";
			return 0;
		}
		zKlient->sendeEncrypted("\1", 1);
		int br = b->getBreite();
		int hi = b->getHeight();
		zKlient->sendeEncrypted((char*)&br, 4);
		zKlient->sendeEncrypted((char*)&hi, 4);
		char* buffer = (char*)b->getBuffer();
		__int64 gr = br * hi * 4;
		for (int i = 0, l = gr > 2048 ? 2048 : (int)gr; gr > 0; i += l, gr -= l, l = gr > 2048 ? 2048 : (int)gr)
			zKlient->sende(&(buffer[i]), l);
		b->release();
		return 1;
	}
	case 0x5: // Beschreibung speichern
	{
		zKlient->sendeEncrypted("\1", 1);
		Text p = pfad.getText();
		p += "/beschreibung.ksgs";
		Datei d;
		d.setDatei(p);
		if (!d.existiert())
			d.erstellen();
		bool ok = d.open(Datei::Style::schreiben);
		char* buffer = new char[2048];
		int gr = 0;
		zKlient->getNachrichtEncrypted((char*)&gr, 4);
		while (gr > 0)
		{
			int l = gr > 2048 ? 2048 : gr;
			zKlient->getNachricht(buffer, l);
			d.schreibe(buffer, l);
			gr -= l;
		}
		delete[] buffer;
		d.close();
		if (!ok)
		{
			fehler = "Es ist ein Fehler beim schreiben der Beschreibung aufgetreten.";
			return 0;
		}
		zKlient->sendeEncrypted("\1", 1);
		return 1;
	}
	case 0x6: // Titelbild speichern
	{
		zKlient->sendeEncrypted("\1", 1);
		Text p = pfad.getText();
		p += "/titel.ltdb";
		int br = 0;
		int hi = 0;
		zKlient->getNachrichtEncrypted((char*)&br, 4);
		zKlient->getNachrichtEncrypted((char*)&hi, 4);
		__int64 gr = br * hi * 4;
		Bild* b = new Bild();
		b->neuBild(br, hi, 0);
		char* buffer = (char*)b->getBuffer();
		for (int i = 0, l = gr > 2048 ? 2048 : (int)gr; gr > 0; i += l, gr -= l, l = gr > 2048 ? 2048 : (int)gr)
			zKlient->getNachricht(&(buffer[i]), l);
		LTDBDatei dat;
		dat.setDatei(dynamic_cast<Text*>(p.getThis()));
		if (!DateiExistiert(p))
			dat.erstellen();
		dat.leseDaten(0);
		dat.remove(0, dat.zBildListe()->get(0));
		if (dat.speichern(0, b, new Text("titel")) < 0)
		{
			fehler = "Fehler beim speichern des Titelbildes.";
			return 0;
		}
		zKlient->sendeEncrypted("\1", 1);
		return 1;
	}
	case 0x7: // Minimap speichern
	{
		zKlient->sendeEncrypted("\1", 1);
		Text p = pfad.getText();
		p += "/minimap.ltdb";
		int br = 0;
		int hi = 0;
		zKlient->getNachrichtEncrypted((char*)&br, 4);
		zKlient->getNachrichtEncrypted((char*)&hi, 4);
		__int64 gr = br * hi * 4;
		Bild* b = new Bild();
		b->neuBild(br, hi, 0);
		char* buffer = (char*)b->getBuffer();
		for (int i = 0, l = gr > 2048 ? 2048 : (int)gr; gr > 0; i += l, gr -= l, l = gr > 2048 ? 2048 : (int)gr)
			zKlient->getNachricht(&(buffer[i]), l);
		LTDBDatei dat;
		dat.setDatei(dynamic_cast<Text*>(p.getThis()));
		if (!DateiExistiert(p))
			dat.erstellen();
		dat.leseDaten(0);
		dat.remove(0, dat.zBildListe()->get(0));
		if (dat.speichern(0, b, new Text("vorschau")) < 0)
		{
			fehler = "Fehler beim speichern des Minimap Bildes.";
			return 0;
		}
		zKlient->sendeEncrypted("\1", 1);
		return 1;
	}
	case 0x8: // Ladebild speichern
	{
		zKlient->sendeEncrypted("\1", 1);
		Text p = pfad.getText();
		p += "/ladebild.ltdb";
		int br = 0;
		int hi = 0;
		zKlient->getNachrichtEncrypted((char*)&br, 4);
		zKlient->getNachrichtEncrypted((char*)&hi, 4);
		__int64 gr = br * hi * 4;
		Bild* b = new Bild();
		b->neuBild(br, hi, 0);
		char* buffer = (char*)b->getBuffer();
		for (int i = 0, l = gr > 2048 ? 2048 : (int)gr; gr > 0; i += l, gr -= l, l = gr > 2048 ? 2048 : (int)gr)
			zKlient->getNachricht(&(buffer[i]), l);
		LTDBDatei dat;
		dat.setDatei(dynamic_cast<Text*>(p.getThis()));
		if (!DateiExistiert(p))
			dat.erstellen();
		dat.leseDaten(0);
		dat.remove(0, dat.zBildListe()->get(0));
		if (dat.speichern(0, b, new Text("spielladen")) < 0)
		{
			fehler = "Fehler beim speichern des Ladebildes.";
			return 0;
		}
		zKlient->sendeEncrypted("\1", 1);
		return 1;
	}
	default:
		fehler = "Unbekannte Nachricht. Eventuell ist der Client nicht auf dem neusten Stand.";
	}
	return 0;
}

// constant
const char* BeschreibungEditor::getLetzterFehler() const
{
	return fehler;
}